@@ -86,7 +86,10 @@ class BleakGATTCharacteristic(abc.ABC):
         Gets the maximum size in bytes that can be used for the *data* argument
         of :meth:`BleakClient.write_gatt_char()` when ``response=False``.
 
-        .. versionadded:: 0.16.0
+        .. warning:: Linux quirk: For BlueZ versions < 5.62, this property
+            will always return ``20``.
+
+        .. versionadded:: 0.16
         """
         return self._max_write_without_response_size
 
@@ -104,7 +107,7 @@ class BleakGATTCharacteristic(abc.ABC):
         raise NotImplementedError()
 
     @abc.abstractmethod
-    def add_descriptor(self, descriptor: BleakGATTDescriptor):
+    def add_descriptor(self, descriptor: BleakGATTDescriptor) -> None:
         """Add a :py:class:`~BleakGATTDescriptor` to the characteristic.
 
         Should not be used by end user, but rather by `bleak` itself.
