@@ -4,14 +4,13 @@ import asyncio
 import logging
 import sys
 import warnings
-from typing import List, Optional
+from typing import List, Literal, Optional
 
-if sys.version_info[:2] < (3, 8):
-    from typing_extensions import Literal
+if sys.version_info < (3, 11):
+    from async_timeout import timeout as async_timeout
 else:
-    from typing import Literal
+    from asyncio import timeout as async_timeout
 
-import async_timeout
 from android.broadcast import BroadcastReceiver
 from android.permissions import Permission, request_permissions
 from jnius import cast, java_method
@@ -59,10 +58,10 @@ class BleakScannerP4Android(BaseBleakScanner):
         self.__javascanner = None
         self.__callback = None
 
-    def __del__(self):
+    def __del__(self) -> None:
         self.__stop()
 
-    async def start(self):
+    async def start(self) -> None:
         if BleakScannerP4Android.__scanner is not None:
             raise BleakError("A BleakScanner is already scanning on this adapter.")
 
@@ -137,7 +136,7 @@ class BleakScannerP4Android(BaseBleakScanner):
         self.__javascanner.flushPendingScanResults(self.__callback.java)
 
         try:
-            async with async_timeout.timeout(0.2):
+            async with async_timeout(0.2):
                 await scanfuture
         except asyncio.exceptions.TimeoutError:
             pass
@@ -209,7 +208,7 @@ class BleakScannerP4Android(BaseBleakScanner):
 
                 return await self.start()
 
-    def __stop(self):
+    def __stop(self) -> None:
         if self.__javascanner is not None:
             logger.debug("Stopping BTLE scan")
             self.__javascanner.stopScan(self.__callback.java)
@@ -218,15 +217,15 @@ class BleakScannerP4Android(BaseBleakScanner):
         else:
             logger.debug("BTLE scan already stopped")
 
-    async def stop(self):
+    async def stop(self) -> None:
         self.__stop()
 
-    def set_scanning_filter(self, **kwargs):
+    def set_scanning_filter(self, **kwargs) -> None:
         # If we do end up implementing this, this should accept List<ScanFilter>
         # and ScanSettings java objects to pass to startScan().
         raise NotImplementedError("not implemented in Android backend")
 
-    def _handle_scan_result(self, result):
+    def _handle_scan_result(self, result) -> None:
         native_device = result.getDevice()
         record = result.getScanRecord()
 
@@ -244,10 +243,10 @@ class BleakScannerP4Android(BaseBleakScanner):
             entry.getKey().toString(): bytes(entry.getValue())
             for entry in record.getServiceData().entrySet()
         }
-        tx_power = result.getTxPower()
+        tx_power = record.getTxPowerLevel()
 
         # change "not present" value to None to match other backends
-        if tx_power == defs.ScanResult.TX_POWER_NOT_PRESENT:
+        if tx_power == -2147483648:  # Integer#MIN_VALUE
             tx_power = None
 
         advertisement = AdvertisementData(
@@ -267,10 +266,7 @@ class BleakScannerP4Android(BaseBleakScanner):
             advertisement,
         )
 
-        if not self._callback:
-            return
-
-        self._callback(device, advertisement)
+        self.call_detection_callbacks(device, advertisement)
 
 
 class _PythonScanCallback(utils.AsyncJavaCallbacks):
